=== modified file 'database/schema/security.cfg'
--- database/schema/security.cfg	2011-12-14 20:03:40 +0000
+++ database/schema/security.cfg	2011-12-18 08:14:12 +0000
@@ -1115,7 +1115,7 @@
 public.bugjob                           = SELECT, INSERT
 public.bugmessage                       = SELECT, INSERT, UPDATE
 public.bugmute                          = SELECT, INSERT, UPDATE, DELETE
-public.bugnomination                    = SELECT, INSERT, UPDATE
+public.bugnomination                    = SELECT, INSERT, UPDATE, DELETE
 public.bugsubscription                  = SELECT, INSERT, UPDATE, DELETE
 public.bugsubscriptionfilter            = SELECT, INSERT, UPDATE, DELETE
 public.bugsubscriptionfilterimportance  = SELECT, INSERT, UPDATE, DELETE

=== modified file 'lib/lp/bugs/browser/bugtask.py'
--- lib/lp/bugs/browser/bugtask.py	2011-12-20 02:00:43 +0000
+++ lib/lp/bugs/browser/bugtask.py	2011-12-20 03:37:28 +0000
@@ -1790,7 +1790,7 @@
     def next_url(self):
         """Return the next URL to call when this call completes."""
         if not self.request.is_ajax:
-            return super(BugTaskDeletionView, self).next_url
+            return self._next_url or self._return_url
         return None
 
     @action('Delete', name='delete_bugtask')
@@ -1801,6 +1801,9 @@
         success_message = ("This bug no longer affects %s."
                     % bugtask.bugtargetdisplayname)
         error_message = None
+        # We set the next_url here before the bugtask is deleted since later
+        # the bugtask will not be available if required to construct the url.
+        self._next_url = self._return_url
 
         try:
             bugtask.delete()

=== modified file 'lib/lp/bugs/browser/tests/test_bugtask.py'
--- lib/lp/bugs/browser/tests/test_bugtask.py	2011-12-19 12:22:48 +0000
+++ lib/lp/bugs/browser/tests/test_bugtask.py	2011-12-20 03:32:24 +0000
@@ -889,14 +889,21 @@
         # Test that the delete action works as expected.
         bug = self.factory.makeBug()
         bugtask = self.factory.makeBugTask(bug=bug)
+        bugtask_url = canonical_url(bugtask, rootsite='bugs')
         target_name = bugtask.bugtargetdisplayname
         with FeatureFixture(DELETE_BUGTASK_ENABLED):
             login_person(bugtask.owner)
             form = {
                 'field.actions.delete_bugtask': 'Delete',
                 }
+            extra = {
+                'HTTP_REFERER': bugtask_url,
+                }
+            server_url = canonical_url(
+                getUtility(ILaunchpadRoot), rootsite='bugs')
             view = create_initialized_view(
-                bugtask, name='+delete', form=form, principal=bugtask.owner)
+                bugtask, name='+delete', form=form, server_url=server_url,
+                principal=bugtask.owner, **extra)
             self.assertEqual([bug.default_bugtask], bug.bugtasks)
             notifications = view.request.response.notifications
             self.assertEqual(1, len(notifications))

=== modified file 'lib/lp/bugs/configure.zcml'
--- lib/lp/bugs/configure.zcml	2011-12-18 14:53:27 +0000
+++ lib/lp/bugs/configure.zcml	2011-12-20 02:25:12 +0000
@@ -952,6 +952,10 @@
                 attributes="
                     approve
                     decline"/>
+            <require
+                permission="launchpad.Edit"
+                attributes="
+                    destroySelf"/>
         </class>
         <securedutility
             class="lp.bugs.model.bugnomination.BugNominationSet"

=== modified file 'lib/lp/bugs/model/bugtask.py'
--- lib/lp/bugs/model/bugtask.py	2011-12-18 14:29:19 +0000
+++ lib/lp/bugs/model/bugtask.py	2011-12-20 02:25:12 +0000
@@ -683,6 +683,17 @@
         self.destroySelf()
         del get_property_cache(bug).bugtasks
 
+        # When a task is deleted, we also delete it's BugNomination entry
+        # if there is one. Sadly, getNominationFor() can return None or
+        # raise NotFoundError so we need to check for both.
+        try:
+            nomination = bug.getNominationFor(target)
+            if nomination is not None:
+                nomination.destroySelf()
+        except NotFoundError:
+            # We don't care if there isn't a nomination
+            pass
+
         # When a task is deleted the bug's heat needs to be recalculated.
         target.recalculateBugHeatCache()
 

=== modified file 'lib/lp/bugs/model/tests/test_bugtask.py'
--- lib/lp/bugs/model/tests/test_bugtask.py	2011-12-12 01:08:21 +0000
+++ lib/lp/bugs/model/tests/test_bugtask.py	2011-12-20 03:32:24 +0000
@@ -1552,14 +1552,20 @@
             self.assertRaises(CannotDeleteBugtask, bugtask.delete)
 
     def test_delete_bugtask(self):
-        # A bugtask can be deleted.
-        bug = self.factory.makeBug()
-        bugtask = self.factory.makeBugTask(bug=bug)
-        bug = bugtask.bug
-        login_person(bugtask.owner)
+        # A bugtask can be deleted and after deletion, re-nominated.
+        owner = self.factory.makePerson()
+        product = self.factory.makeProduct(driver=owner, bug_supervisor=owner)
+        bug = self.factory.makeBug(
+            product=product, owner=owner)
+        target = self.factory.makeProductSeries(product=product)
+        login_person(bug.owner)
+        nomination = bug.addNomination(bug.owner, target)
+        nomination.approve(bug.owner)
+        bugtask = bug.getBugTask(target)
         with FeatureFixture(self.flags):
             bugtask.delete()
         self.assertEqual([bug.default_bugtask], bug.bugtasks)
+        self.assertTrue(bug.canBeNominatedFor(target))
 
     def test_delete_default_bugtask(self):
         # The default bugtask can be deleted.

